
# q-expansion of newform 1520.4.a.c, downloaded from the LMFDB on 03 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1520
weight = 4
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-4], [5], [32], [12], [-42], [114], [-19], [-160], [214], [144], [94], [-6], [308], [-184], [-274], [-276], [-826], [-52], [344], [-166], [688], [-996], [1578], [786], [126], [368], [-204], [-1434], [1218], [-904], [2180], [-2566], [-1988], [1134], [2440], [3238], [1420], [2336], [1206], [1412], [3742], [1472], [1458], [2046], [1496], [-844], [3400], [-1844], [-1090], [2842], [2400], [2130], [2364], [6290], [-8112], [-4794], [-304], [2062], [-4054], [-7996], [-3906], [1820], [-712], [9130], [6342], [4748], [5154], [12148], [-602], [2], [-1960], [5864], [558], [4876], [424], [-9890], [-234], [11602], [-14806], [-6252], [-10482], [-3936], [10946], [7800], [11364], [7330], [-12774], [-3786], [19448], [-4596], [-12432], [18016], [-1972], [-18780], [-8256], [-13290], [7610], [13636], [1350], [20396], [-4458], [-18228], [-8246], [924], [16322], [-17500], [-1486], [-24616], [-13334], [12056], [-98], [2938], [-25316], [-1256], [20290], [-10676], [-11264], [25878], [-1500], [-7618], [-4110], [-21474], [-4668], [-13292], [7206], [-5666], [0], [4048], [12134], [13556], [-4368], [-3872], [6190], [-7062], [-5438], [1182], [-12452], [15526], [31034], [34636], [-20082], [-33568], [-19644], [726], [-3512], [-39442], [-40454], [3436], [10056], [-44394], [-18222], [29404], [-18576], [2644], [-39744], [19720], [36258], [14586], [30182], [-15828], [22746], [-16480], [16940], [-40062], [-4832], [4144], [35294], [4690], [-8498], [-44980], [-45930], [34504], [9018], [-27344], [19018], [23996], [-18242], [38032], [-8234], [-1832], [12484], [-41314], [-69558], [-13224], [-50], [29862], [37532], [-10470], [-34336], [20498], [2036], [-61356], [-42218], [20284], [-58742], [-48110], [51942], [-48190], [34480], [-54906], [-17920], [-7570], [37890], [-42180], [41862], [49488], [1980], [-34854], [-65044], [38674], [15438], [33568], [50660], [-72696], [46970], [-47064], [45330], [-38928], [5126], [72814], [7544], [22274], [7416], [-54596], [23054], [-53958], [-95136], [34496], [-56948], [48182], [13668], [56112], [-71750], [73460], [-79800], [35826], [18606], [45996], [66680], [-38068], [18972], [17360], [-61978], [-64542], [-98104], [9816], [102340], [96732], [63192], [-117530], [-94014], [-46880], [-91046], [-63882], [-9468], [-60786], [74628], [-57442], [41610], [31048], [38668], [89278], [-31738], [-638], [70356], [-54426], [92394], [8276], [17630], [105862], [-130212], [94346], [104496], [-27646], [140832], [-100748], [-63434], [-119174], [-42220], [-89336], [-65176], [50224], [2350], [4228], [86032], [-4270], [90798], [-29016], [96098], [-7242], [128556], [107066], [17420], [-164522], [99110], [-37168], [-38482], [60172], [45948], [29242], [-43338], [-164320], [-80052], [-108260], [167570], [-73676], [88166], [-169880], [-114018], [-111944], [-51730], [-112638], [3484], [74976], [38106], [-63708], [-37968], [-31038], [-124622], [-11052], [-10838], [95926], [-47352], [-18022], [-111662], [-124620], [37908], [34040], [-118946], [-102634], [30598], [-7504], [-69524], [-135124], [-47628], [96630], [-104334], [103354], [-121528], [-160850], [216906], [-164274], [-38056], [71558], [92724], [-121858], [-3372], [38432], [167118], [-59388], [-99574], [-100938], [-194696], [82910], [-94534], [117008], [16492], [80274], [-194784], [23550], [-12198], [-76280], [112428], [77564], [154442], [32374], [199568], [32074], [-116828], [230204], [-37160], [-90786], [44056], [193190], [-13308], [-127808], [-100702], [145490], [-116934], [13302], [223002], [-12944], [-219806], [188148], [-47248], [-250576], [214446], [251396], [-70152], [144066], [-120482], [104700], [170860], [242600], [-35846], [-147200], [120762], [128780], [134126], [-133194], [224226], [228040], [-39686], [33102], [-282712], [62902], [-1614], [242988], [-107564], [-35646], [-246066], [284276], [97924], [-31750], [-4650], [61184], [-78624], [-19406], [124416], [-300042], [-85346], [-101720], [-270132], [-299286], [-35402], [-130356], [-42742], [181276], [18280]]
